-------------------------------------------------------------------------------
--                G  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local MAX_TRADE_SKILLS = 6;

-- Mapping of spell ID to trade skill ID
local gatherSpellSkill =
{
  -- Archaeology
  [ 73979] = 794,
  -- Engineering
  [ 49383] = 202,
  -- Fishing
  [  7620] = 356,
  [  7731] = 356,
  [  7732] = 356,
  [ 18248] = 356,
  [ 33095] = 356,
  [ 51294] = 356,
  [ 88868] = 356,
  [110410] = 356,
  [131474] = 356,
  [131476] = 356,
  [131490] = 356,
  -- Herbalism
  [  2366] = 182,
  [  2368] = 182,
  [  3570] = 182,
  [ 11993] = 182,
  [ 28695] = 182,
  [ 50300] = 182,
  [ 74519] = 182,
  [110413] = 182,
  -- Mining
  [  2575] = 186,
  [  2576] = 186,
  [  3564] = 186,
  [ 10248] = 186,
  [ 29354] = 186,
  [ 50310] = 186,
  [ 74517] = 186,
  [102161] = 186,
  -- Skinning
  [  8613] = 393,
  [  8617] = 393,
  [  8618] = 393,
  [ 10768] = 393,
  [ 32678] = 393,
  [ 50305] = 393,
  [ 74522] = 393,
  [102216] = 393,
};

-- Info about the gathering skills
local gatherSkillInfo =
{
  [794] = -- Archaeology
  {
    index   = 0,
    tooltip = L["Requires Archaeology"]
  },
  [202] = -- Engineering
  {
    index   = 0,
    tooltip = UNIT_SKINNABLE_BOLTS
  },
  [356] = -- Fishing
  {
    index   = 0,
    tooltip = ""
  },
  [182] = -- Herbalism
  {
    index   = 0,
    tooltip = UNIT_SKINNABLE_HERB
  },
  [186] = -- Mining
  {
    index   = 0,
    tooltip = UNIT_SKINNABLE_ROCK
  },
  [393] = -- Skinning
  {
    index   = 0,
    tooltip = UNIT_SKINNABLE_LEATHER
  },
};

local eventFrame;

--
-- These value have been copied from 'Blizzard_TradeSkillUI.lua' because the
-- Trade Skill panel is implemented as an on-demand addon.
--
local tradeSkillTypeInfo =
{
  ["trivial"] = {r = 0.50, g = 0.50, b = 0.50},
  ["easy"]    = {r = 0.25, g = 0.75, b = 0.25},
  ["medium"]  = {r = 1.00, g = 1.00, b = 0.00},
  ["optimal"] = {r = 1.00, g = 0.50, b = 0.25},
};

local tradeSkillInfo =
{
  professionIdx  = 0,
  recipeIdx      = 0,
  maxRank        = 0,
  numProfessions = 0,
  numMaxed       = 0,
};

local recipeInfo =
{
  index     = 0,
  spellId   = 0,
  skillType = "",
  rank      = 0,
  maxRank   = 0,
  skillName = "",
};

local tradeSkillFrameInfo =
{
  buttons      = {},
  textures     = {},
  searchPoints = {},
  searchWidth  = 0,
};

local recipeHistory = {};
local recipeItems = {};

-------------------------------------------------------------------------------
--             L  O  C  A  L     D  E  F  I  N  I  T  I  O  N  S  
-------------------------------------------------------------------------------

local tradeSkillClearFilters;
local tradeSkillFrameOnShow;
local tradeSkillTypeFromTooltip;

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to enable/disable the recipe history buttons
--
local function recipeHistoryBtnEnable(tradeSkillIdx)
  -- Enable/disable the 'Back' button
  if (tradeSkillFrameInfo.buttons[1] ~= nil) then
    tradeSkillFrameInfo.buttons[1]:SetEnabled(
      recipeHistory[tradeSkillIdx]:IsMoveValid(true));
  end
    
  -- Enable/disable the 'Forward' button
  if (tradeSkillFrameInfo.buttons[2] ~= nil) then
    tradeSkillFrameInfo.buttons[2]:SetEnabled(
      recipeHistory[tradeSkillIdx]:IsMoveValid(false));
  end

  return;
end

--
-- Function call when the Recipe History Back or Forward buttons are clicked.
--
local function recipeHistoryBtnOnClick(self, mouseButton)
  -- Local Variables
  local recipeName;
  local recipeIdx = 0;

  -- Work out which button has been clicked
  if (self:GetID() == 1) then
    recipeName = recipeHistory[tradeSkillInfo.professionIdx]:Back();
  elseif (self:GetID() == 2) then
    recipeName = recipeHistory[tradeSkillInfo.professionIdx]:Forward();
  end

  -- Check there was a recipe name returned
  if (recipeName ~= nil) then
    -- Clear all filtering
    tradeSkillClearFilters();
    
    -- Search for the recipe's index
    for idx = 1, GetNumTradeSkills() do
      if (GetTradeSkillInfo(idx) == recipeName) then
        recipeIdx = idx;
        break;
      end
    end
  end

  -- Check the recipe was found
  if (recipeIdx > 0) then
    -- Select the recipe
    TradeSkillFrame_SetSelection(recipeIdx);
    TradeSkillFrame_Update();
  end

  -- Update the state of the recipe history buttons
  recipeHistoryBtnEnable(tradeSkillInfo.professionIdx);

  -- Update the Trade Skill filter bar
  TradeSkillUpdateFilterBar();

  return;
end

--
-- Function to make sure the recipe index in the 'recipeItems' list for an
-- item is valid. It checks that the item created by the recipe matches the
-- specified item name. This function is defensive, to deal with situations
-- where the recipe list has become corrupted for some reason, usually because
-- it was built before the trade skill is truly ready.
--
local function recipeItemsIsIndexValid(itemName)
  -- Local Variables
  local itemLink;
  local recipeIdx;
  local isValid = false;

  -- Check at least one recipe is known for the item
  for idx = 1, 2 do
    -- Get the recipe index for the item
    recipeIdx = recipeItems[itemName];

    -- Check if only one recipe is known for the item
    if ((recipeIdx ~= nil) and (recipeIdx > 0)) then
      itemLink = GetTradeSkillItemLink(recipeIdx);
      if (string.match((itemLink or ""), "|h%[(.*)%]|h|r") == itemName) then
        isValid = true;
        break;
      end
    elseif (recipeIdx == -1) then
      isValid = true;
      break;
    elseif (idx == 1) then
      -- Update the recipe items list, because it appears invalid
      recipeItemsUpdate(false)
    end
  end

  return isValid;
end

--
-- Function to update the list of items that can be created by known recipes
-- for the current trade skill
--
local function recipeItemsUpdate(restoreFilters)
  -- Local Variables
  local itemLink;
  local itemName;
  local itemNameFilter = GetTradeSkillItemNameFilter();

  -- Check if the trade skill isn't ready to be interrogated
  if (not IsTradeSkillReady()) then
    -- Create a timer to update the list
    NigglesTimerCreate("RecipeItemsUpdate", recipeItemsUpdate,
      restoreFilters, nil, 0.25, false);
    return false;
  end
  
  -- Clear the current filter settings
  tradeSkillClearFilters();

  -- Get the list of items that can be created by known recipes
  wipe(recipeItems);
  for idx = 1, GetNumTradeSkills() do
    -- Get the name of the item created by the recipe
    itemLink = GetTradeSkillItemLink(idx);
    itemName = string.match((itemLink or ""), "|h%[(.*)%]|h|r");
    if (itemName ~= nil) then
      -- Check the item hasn't already been seen
      if (recipeItems[itemName] == nil) then
        -- Save the item's name and the recipe's index
        recipeItems[itemName] = idx;
      else
        -- Set to a special value to indicate multiple recipes create the item
        recipeItems[itemName] = -1;
      end
    end
  end

  -- Restore the filter settings, if required
  if (restoreFilters) then
    TradeSkillFrameSearchBox:SetText(itemNameFilter or "");
    TradeSkillFrameSearchBox:GetScript("OnEditFocusLost")(
      TradeSkillFrameSearchBox);
    TradeSkillOnlyShowSkillUps(TradeSkillFrame.filterTbl.hasSkillUp);
    TradeSkillOnlyShowMakeable(TradeSkillFrame.filterTbl.hasMaterials);
    SetTradeSkillCategoryFilter(TradeSkillFrame.filterTbl.subClassValue, 0);
    SetTradeSkillInvSlotFilter(TradeSkillFrame.filterTbl.slotValue, 1, 1);
  end
  TradeSkillUpdateFilterBar();
 
  return true;
end

--
-- Function called when a recipe reagent is clicked. It checks if the reagent
-- can be created by the current trade skill and is known by the current
-- character. If it is, the recipe is selected.
--
local function recipeReagentBtnOnClick(self, mouseButton)
  -- Local Variables
  local recipeIdx = (TradeSkillFrame.selectedSkill or 0);
  local recipeName = GetTradeSkillInfo(recipeIdx);
  local reagentName;
 
  -- Check if the button click should be processed
  if ((mouseButton == "LeftButton") and
      (not IsModifierKeyDown()) and
      (recipeIdx > 0)) then
    -- Get the reagent's name
    reagentName = GetTradeSkillReagentInfo(recipeIdx, self:GetID());
    if (recipeItems[reagentName] ~= nil) then
      -- Clear all filtering
      tradeSkillClearFilters();
      
      -- Check the reagent's recipe index is valid
      if (recipeItemsIsIndexValid(reagentName)) then
        -- Make sure the current recipe is in the trade skill's recipe history
        recipeHistory[tradeSkillInfo.professionIdx]:Add(recipeName);

        -- Check if the reagent has a single recipe
        if ((recipeItems[reagentName] > 0) and 
            (recipeItems[reagentName] <= GetNumTradeSkills())) then
          TradeSkillFrame_SetSelection(recipeItems[reagentName]);
          TradeSkillFrame_Update();
          recipeHistory[tradeSkillInfo.professionIdx]:Add(reagentName);
          recipeHistoryBtnEnable(tradeSkillInfo.professionIdx);
        elseif (recipeItems[reagentName] == -1) then
          -- Set the reagent as the search string
          TradeSkillFrameSearchBox:SetText(reagentName);
        end
      end

      -- Update the Trade Skill filter bar
      TradeSkillUpdateFilterBar();
    end
  end

  return;
end

--
-- Function hooked to 'TradeSkillFrame_SetSelection'. It shows/hides the 
-- textures used to indicate recipe reagents that can be made by the current
-- trade skill and character.
--
local function recipeReagentSetSelection()
  -- Local Variables
  local button;
  local recipeIdx = (TradeSkillFrame.selectedSkill or 0);
  local textures = tradeSkillFrameInfo.textures;
  local reagentName;

  -- Check there is a recipe displayed
  if ((TradeSkillFrame:IsShown()) and (recipeIdx > 0)) then
    -- Show/Hide the textures that indicate a reagent can be created
    for idx = 1, GetTradeSkillNumReagents(recipeIdx) do
      button = _G["TradeSkillReagent"..idx]
      if ((button ~= nil) and
          (button:IsShown()) and
          (textures[idx] ~= nil)) then
        reagentName = GetTradeSkillReagentInfo(recipeIdx, idx);
        textures[idx]:SetShown(
          ((reagentName ~= nil) and (recipeItems[reagentName] ~= nil)));
      end
    end
  end
  
  return;
end

-- 
-- Function to clear all filters for the Trade Skill panel
--
tradeSkillClearFilters = function()
  -- Clear all trade skill filters
  TradeSkillFrameSearchBox:ClearFocus();
  TradeSkillFrameSearchBox:SetText("");
  TradeSkillFrameSearchBox:GetScript("OnEditFocusLost")(
    TradeSkillFrameSearchBox);
  SetTradeSkillItemNameFilter();
  TradeSkillOnlyShowSkillUps(false);
  TradeSkillOnlyShowMakeable(false);
  SetTradeSkillCategoryFilter(-1, 0);
  SetTradeSkillInvSlotFilter(-1, 1, 1);

  return;
end

--
-- Function hooked to 'DoTradeSkill' to capture the spell ID of the item being
-- created so that its skill type can be obtained for each iteration
--
local function tradeSkillDoRecipe(idx, count)
  -- Local Variables
  local spellId;

  -- Extract the spell ID from the recipe's link
  spellId = string.gsub(GetTradeSkillRecipeLink(idx),
    "^[^:]*:([^|]*)\|h.*$", "%1");
  if (spellId ~= nil) then
    -- Save the spell ID
    recipeInfo.index     = tonumber(idx);
    recipeInfo.spellId   = tonumber(spellId);
    recipeInfo.skillName = GetTradeSkillLine();
  else
    recipeInfo.index     = 0;
    recipeInfo.spellId   = 0;
    recipeInfo.skillName = "";
  end

  return;
end

--
-- Function called when the Trade Skill frame is shown. It enables/disables
-- Niggles features, depending on the current settings.
--
tradeSkillFrameOnShow = function(self)
  -- Local Variables
  local button;
  local texturePath;
  local textures = tradeSkillFrameInfo.textures;
  local idx = 1;

  -- Set up the recipe history buttons
  for idx, suffix in ipairs({"PrevPage", "NextPage"}) do
    -- Create the button, if required
    if ((NigglesDBC.settings.ProfGeneralHistory) and
        (tradeSkillFrameInfo.buttons[idx] == nil)) then
      -- Work out the path for the button's textures
      texturePath = "Interface\\Buttons\\UI-SpellbookIcon-"..suffix;

      -- Create the button
      button = CreateFrame("Button", nil, TradeSkillFrame);
      button:SetWidth(23);
      button:SetHeight(22);
      button:SetNormalTexture(texturePath.."-Up");
      button:SetPushedTexture(texturePath.."-Down");
      button:SetDisabledTexture(texturePath.."-Disabled");
      button:SetHighlightTexture(
        "Interface\\Buttons\\UI-Common-MouseHilight", "ADD");
      button:SetPoint("TOPLEFT", TradeSkillRankFrame, "BOTTOMLEFT",
        5+((idx-1)*20), -8);
      button:SetID(idx);
      button:SetScript("OnClick", recipeHistoryBtnOnClick);
      button:Disable();
      tradeSkillFrameInfo.buttons[idx] = button;
    else
      button = tradeSkillFrameInfo.buttons[idx];
    end

    -- Show/Hide the button
    if (button ~= nil) then
      button:SetShown(NigglesDBC.settings.ProfGeneralHistory);
    end
  end

  -- Adjust the position of the search box
  if (NigglesDBC.settings.ProfGeneralHistory) then
    TradeSkillFrameSearchBox:SetPoint("TOPLEFT",
      tradeSkillFrameInfo.buttons[2], "TOPRIGHT", 9, -1);
    TradeSkillFrameSearchBox:SetWidth(131);
  else
    NigglesFramePointsSet(TradeSkillFrameSearchBox,
      tradeSkillFrameInfo.searchPoints);
    TradeSkillFrameSearchBox:SetWidth(tradeSkillFrameInfo.searchWidth);
  end
  
  -- Create the textures that indicate creatable recipe reagents, if required
  if (NigglesDBC.settings.ProfGeneralReagents) then
    while (_G["TradeSkillReagent"..idx] ~= nil) do
      button = _G["TradeSkillReagent"..idx];
      if (textures[idx] == nil) then
        textures[idx] =
          _G["TradeSkillReagent"..idx]:CreateTexture(nil, "OVERLAY");
        textures[idx]:SetTexture("Interface\\SPELLBOOK\\RotationIconFrame");
        textures[idx]:SetPoint("TOPLEFT", button.icon, -6, 6);
        textures[idx]:SetPoint("BOTTOMRIGHT", button.icon, 7, -7);
      end
      idx = idx+1;
    end
  end
  
  -- Set up the textures for recipe reagents
  recipeReagentSetSelection()

  return;
end

--
-- Function to process events
--
local function tradeSkillOnEvent(self, event, ...)
  -- Local Variables
  local skillId;

  -- Process the event
  if ((event == "UNIT_SPELLCAST_START") or
      (event == "UNIT_SPELLCAST_CHANNEL_START")) then
    -- Local Variables
    local skillId;

    -- Get the spell ID
    local _, _, _, _, spellId = ...;
    skillId = gatherSpellSkill[spellId];

    -- Check if a gathering skill is being cast
    if ((skillId ~= nil) and (gatherSkillInfo[skillId] ~= nil)) then
      -- Get the skill type from the tooltip
      recipeInfo.skillType = tradeSkillTypeFromTooltip(skillId);

      -- Get the profession's current and max ranks
      recipeInfo.skillName, _, recipeInfo.rank, recipeInfo.maxRank =
        GetProfessionInfo(gatherSkillInfo[skillId].index);
    elseif (spellId == recipeInfo.spellId) then
      -- Get the skill type
      if (recipeInfo.index > 0) then
        -- Get the skill type
        _, recipeInfo.skillType = GetTradeSkillInfo(recipeInfo.index);
        _, recipeInfo.rank, recipeInfo.maxRank = GetTradeSkillLine();
      else
        recipeInfo.skillType = "optimal";
      end
    else
      recipeInfo.skillType = "";
    end

    -- Check if a warning should be displayed
    if ((recipeInfo.skillType ~= "") and
        (recipeInfo.skillType ~= "trivial") and
        (recipeInfo.maxRank < tradeSkillInfo.maxRank) and
        (recipeInfo.rank >= recipeInfo.maxRank-5)) then
      if (recipeInfo.rank >= recipeInfo.maxRank) then
        Niggles:AddWarning(L["Trade Skill Capped"], recipeInfo.skillName);
      else
        Niggles:AddWarning(L["Trade Skill Near Cap"], recipeInfo.skillName);
      end
      PlaySound("RaidWarning");
    end
  elseif (event == "SKILL_LINES_CHANGED") then
    -- Initialise some variables
    local skillRank;
    local professions = {};
    tradeSkillInfo.numProfessions = 0;
    tradeSkillInfo.numMaxed = 0;

    -- Get the character's professions
    professions[1], professions[2], professions[3],
    professions[4], professions[5], professions[6] = GetProfessions();

    -- Get the index of each gather professions the character knows
    for _, professionIdx in pairs(professions) do
      -- Get info for the profession
      _, _, skillRank, _, _, _, skillId = GetProfessionInfo(professionIdx);

      -- Check if the skill is a gather skill
      if (gatherSkillInfo[skillId] ~= nil) then
        -- Save the profession's skill index
        gatherSkillInfo[skillId].index = professionIdx;
      end

      -- Check if the skill is at the maximum rank possible
      tradeSkillInfo.numProfessions = tradeSkillInfo.numProfessions+1;
      if (skillRank == tradeSkillInfo.maxRank) then
        tradeSkillInfo.numMaxed = tradeSkillInfo.numMaxed+1;
      end
    end

    -- Check if there are any professions that aren't maxed
    if (tradeSkillInfo.numMaxed < tradeSkillInfo.numProfessions) then
      eventFrame:RegisterUnitEvent("UNIT_SPELLCAST_START", "player");
      eventFrame:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_START", "player");
    else
      eventFrame:UnregisterEvent("UNIT_SPELLCAST_START");
      eventFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
    end
  elseif (event == "TRADE_SKILL_SHOW") then
    -- Local Variables
    local tradeSkillName = GetTradeSkillLine();

    -- Check a known profession is being shown
    if (tradeSkillName ~= "UNKNOWN") then
      if (IsTradeSkillLinked()) then
        -- Use a special history list for other characters
        tradeSkillInfo.professionIdx = #recipeHistory;
        recipeHistory[tradeSkillInfo.professionIdx]:Clear();
      else
        -- Get the ID of all the character's professions
        local professions = {};
        professions[1], professions[2], professions[3],
        professions[4], professions[5], professions[6] = GetProfessions();

        -- Work out the index of the current profession 
        for idx, id in pairs(professions) do
          if (GetProfessionInfo(id) == tradeSkillName) then
            tradeSkillInfo.professionIdx = idx;
            break;
          end
        end
      end

      -- Update the list of items that can be created by known recipes
      recipeItemsUpdate(true);
    else
      tradeSkillInfo.professionIdx = 0;
    end
    
    -- Enable/disable the recipe history buttons
    -- NOTE: This has to be done here, rather than in the 'OnShow' script hook
    --       because this event happens after the frame is shown.
    recipeHistoryBtnEnable(tradeSkillInfo.professionIdx);
  elseif (event == "SPELLS_CHANGED") then
    if (GetTradeSkillLine() ~= "UNKNOWN") then
      recipeItemsUpdate(true);
    end
  elseif (event == "TRADE_SKILL_HIDE") then
    wipe(recipeItems);
  end

  return;
end

--
-- Function called when the user clicks on a recipe. It adds the name of the
-- recipe to the recipe history for the current trade skill
--
local function tradeSkillRecipeOnClick(self, mouseButton)
  -- Local Variable
  local recipeName;
  local recipeType;

  -- Check if the recipe should be added to the trade skill's recipe history
  recipeName, recipeType = GetTradeSkillInfo(self:GetID());
  if ((NigglesDBC.settings.ProfGeneralHistory) and
      (recipeName ~= nil) and
      (recipeName ~= "") and
      (recipeType ~= "header") and
      (recipeType ~= "subheader")) then
    -- Add the recipe to the trade skill's recipe history
    recipeHistory[tradeSkillInfo.professionIdx]:Add(recipeName);
    recipeHistoryBtnEnable(tradeSkillInfo.professionIdx);
  end

  return;
end

--
-- Function hooked to 'SolveArtifact' to capture the spell ID of the artifact
-- being solved.
--
local function tradeSkillSolveArtifact(...)
  -- Local Variables
  local artifactName = GetSelectedArtifactInfo();

  -- Check the artifact's name could be obtained
  if (artifactName ~= nil) then
    -- Get the spell ID for the artifact
    recipeInfo.index     = 0;
    recipeInfo.spellId   = NigglesArchaeologyGetSpellId(artifactName);
    recipeInfo.skillType = "optimal";
    recipeInfo.skillName, _, recipeInfo.rank, recipeInfo.maxRank =
      GetProfessionInfo(gatherSkillInfo[794].index);
  end

  return;
end

--
-- Function to get the skill type from the GameTooltip when a world object is
-- gathered.  --
tradeSkillTypeFromTooltip = function(skillId)
  -- Local Variables
  local colourR;
  local colourG;
  local colourB;
  local skillType = "optimal";

  -- Process each line of the Game Tooltip
  for idx = GameTooltip:NumLines(), 1, -1 do
    -- Check if the tooltip line matches the gather skill's tooltip
    if (_G["GameTooltipTextLeft"..idx]:GetText() ==
         gatherSkillInfo[skillId].tooltip) then
      -- Get the colour of the tooltip line
      colourR, colourG, colourB = _G["GameTooltipTextLeft"..idx]:GetTextColor();

      -- Find the skill type, based on the colour
      for currentType, currentInfo in pairs(tradeSkillTypeInfo) do
        if ((math.abs(colourR-currentInfo.r) < 0.01) and
            (math.abs(colourG-currentInfo.g) < 0.01) and
            (math.abs(colourB-currentInfo.b) < 0.01)) then
          skillType = currentType;
        end
      end
      break;
    end
  end

  return skillType;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to enable/disable the Niggles trade skill features
--
function NigglesTradeSkillEnable()
  -- Local Variables
  local button;
  local idx = 1;

  -- Check if the frame used to monitor events has already be created
  if (eventFrame == nil) then
    -- Create the event frame
    eventFrame = CreateFrame("FRAME", nil);
    eventFrame:SetScript("OnEvent", tradeSkillOnEvent);

    -- Initialise some info
    tradeSkillInfo.maxRank = PROFESSION_RANKS[#PROFESSION_RANKS][1];

    -- Hook the functions that use trade skill recipes
    hooksecurefunc("DoTradeSkill",  tradeSkillDoRecipe);
    hooksecurefunc("SolveArtifact", tradeSkillSolveArtifact);
  end

  -- (Un)register for events, depending on the current settings
  if (NigglesDBC.settings.ProfGeneralReagents) then
    eventFrame:RegisterEvent("TRADE_SKILL_SHOW");
    eventFrame:RegisterEvent("TRADE_SKILL_HIDE");
    eventFrame:RegisterEvent("SPELLS_CHANGED");
  else
    eventFrame:UnregisterEvent("TRADE_SKILL_SHOW");
    eventFrame:UnregisterEvent("TRADE_SKILL_HIDE");
    eventFrame:UnregisterEvent("SPELLS_CHANGED");
  end
  if (NigglesDBC.settings.WarningTradeSkillCap) then
    eventFrame:RegisterEvent("SKILL_LINES_CHANGED");
    tradeSkillOnEvent(eventFrame, "SKILL_LINES_CHANGED");
  else
    eventFrame:UnregisterEvent("UNIT_SPELLCAST_START");
    eventFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
    eventFrame:UnregisterEvent("SKILL_LINES_CHANGED");
  end

  -- Update the Trade Skill frame, based on current settings
  if ((TradeSkillFrame ~= nil) and 
      (TradeSkillFrame:IsShown())) then
    tradeSkillFrameOnShow(TradeSkillFrame);
  end

  -- Update/Clear the recipe list, based on current settings
  if ((NigglesDBC.settings.ProfGeneralReagents) and
      (GetTradeSkillLine() ~= "UNKNOWN")) then
    recipeItemsUpdate(true);
  else
    -- Clear any recipe list
    wipe(recipeItems);
  end

  return;
end

--
-- Function called when Blizzard's Trade Skill addon is loaded
--
function NigglesTradeSkillOnLoad()
  -- Local Variables
  local button;
  local texture;
  local idx = 1;

  -- Hook the Trade Skill Frame
  TradeSkillFrame:HookScript("OnShow", tradeSkillFrameOnShow);

  -- Hook Blizzard functions
  hooksecurefunc("TradeSkillSkillButton_OnClick", tradeSkillRecipeOnClick);
  hooksecurefunc("TradeSkillFrame_SetSelection", recipeReagentSetSelection);

  -- Hook the 'OnClick' script handler for the reagent buttons
  while (_G["TradeSkillReagent"..idx] ~= nil) do
    button = _G["TradeSkillReagent"..idx];
    button:HookScript("OnClick", recipeReagentBtnOnClick);
    idx = idx+1;
  end

  -- Save the points and width of the Trade Skill frame's search box
  NigglesFramePointsGet(TradeSkillFrameSearchBox,
    tradeSkillFrameInfo.searchPoints);
  tradeSkillFrameInfo.searchWidth = TradeSkillFrameSearchBox:GetWidth();

  -- Initialise the history for each trade skill
  for idx = 1, MAX_TRADE_SKILLS+1 do
    recipeHistory[idx] = NigglesHistory:New();
  end

  return;
end
